local coveredBridge = ...

--------------------------------------------------------------------------

-- Register IDs for all prefab nodes referenced here
coveredBridge:registerAssetId("models/coveredBridge.fbx/Prefab/coveredBridge/CenterPart", "COVERED_BRIDGE_CENTER_PREFAB")
coveredBridge:registerAssetId("models/coveredBridge.fbx/Prefab/coveredBridge/RootPart", "COVERED_BRIDGE_CENTER_ROOT_PREFAB")
coveredBridge:registerAssetId("models/coveredBridge.fbx/Prefab/coveredBridge/StartSlopePart", "COVERED_BRIDGE_SLOPE_START_PREFAB")
coveredBridge:registerAssetId("models/coveredBridge.fbx/Prefab/coveredBridge/EndSlopePart", "COVERED_BRIDGE_SLOPE_END_PREFAB")
coveredBridge:registerAssetId("models/coveredBridge.fbx/Prefab/coveredBridge/StartPart", "COVERED_BRIDGE_START_PREFAB")
coveredBridge:registerAssetId("models/coveredBridge.fbx/Prefab/coveredBridge/EndPart", "COVERED_BRIDGE_END_PREFAB")
coveredBridge:registerAssetId("models/coveredBridge.fbx/Prefab/coveredBridge/TilingPart", "COVERED_BRIDGE_TILING_PREFAB")
coveredBridge:registerAssetId("models/coveredBridgeDecoration.fbx/Prefab/coveredBridge/CenterCoveragePart", "COVERED_BRIDGE_CENTER_COVERAGE_PREFAB")
coveredBridge:registerAssetId("models/coveredBridgeDecoration.fbx/Prefab/coveredBridge/StartEndCoveragePart", "COVERED_BRIDGE_START_END_COVERAGE_PREFAB")
coveredBridge:registerAssetId("models/coveredBridgeDecoration.fbx/Prefab/coveredBridge/HousePart", "COVERED_BRIDGE_HOUSE_PREFAB")
coveredBridge:registerAssetId("models/coveredBridgeDecoration.fbx/Prefab/coveredBridge/TailorPart", "COVERED_BRIDGE_TAILOR_PREFAB")
coveredBridge:registerAssetId("models/coveredBridgeDecoration.fbx/Prefab/coveredBridge/DecorationDoorPart", "COVERED_BRIDGE_DOOR_PREFAB")

--------------------------------------------------------------------------

-- Register bridge tiling (the pillar part that extends to the ground)
coveredBridge:register({
	DataType = "BUILDING_PART",
	Id = "COVERED_BRIDGE_TILING_PART",
	Mover = { DataType = "BUILDING_PART_MOVER" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = "COVERED_BRIDGE_TILING_PREFAB"
	},
	Cost = {
		DataType = "BUILDING_PART_COST",
		RessourcesNeeded = {
			{ Resource = "STONE", Quantity = 1 }
		}
	},
})

-- Register bridge start part
coveredBridge:register({
	DataType = "BUILDING_PART",
	Id = "COVERED_BRIDGE_START_PART",
	Mover = { DataType = "BUILDING_PART_MOVER" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_BASEMENT",
		CoreObjectPrefab = "COVERED_BRIDGE_START_PREFAB",
		FillerList = {
			"COVERED_BRIDGE_TILING_PART"
		},
		BasementScale = 2,
	},
	Cost = {
		DataType = "BUILDING_PART_COST",
		RessourcesNeeded = {
			{ Resource = "STONE", Quantity = 5 }
		},
	},
})

-- Register bridge end part
coveredBridge:register({
	DataType = "BUILDING_PART",
	Id = "COVERED_BRIDGE_END_PART",
	Mover = { DataType = "BUILDING_PART_MOVER_BRIDGE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_BASEMENT",
		CoreObjectPrefab = "COVERED_BRIDGE_END_PREFAB",
		FillerList = {
			"COVERED_BRIDGE_TILING_PART"
		},
		BasementScale = 2,
	},
	Cost = {
		DataType = "BUILDING_PART_COST",
		RessourcesNeeded = {
			{ Resource = "STONE", Quantity = 5 }
		},
	},
})

-- Register bridge start slope part
coveredBridge:register({
	DataType = "BUILDING_PART",
	Id = "COVERED_BRIDGE_SLOPE_START_PART",
	Mover = { DataType = "BUILDING_PART_MOVER" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_BASEMENT",
		CoreObjectPrefab = "COVERED_BRIDGE_SLOPE_START_PREFAB",
		FillerList = {
			"COVERED_BRIDGE_TILING_PART"
		},
		BasementScale = 2,
	},
	Cost = {
		DataType = "BUILDING_PART_COST",
		RessourcesNeeded = {
			{ Resource = "STONE", Quantity = 1 }
		},
	},
})

-- Register bridge end slope part
coveredBridge:register({
	DataType = "BUILDING_PART",
	Id = "COVERED_BRIDGE_SLOPE_END_PART",
	Mover = { DataType = "BUILDING_PART_MOVER" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_BASEMENT",
		CoreObjectPrefab = "COVERED_BRIDGE_SLOPE_END_PREFAB",
		FillerList = {
			"COVERED_BRIDGE_TILING_PART"
		},
		BasementScale = 2,
	},
	Cost = {
		DataType = "BUILDING_PART_COST",
		RessourcesNeeded = {
			{ Resource = "STONE", Quantity = 1 }
		},
	},
})

-- Register bridge center top part
coveredBridge:register({
	DataType = "BUILDING_PART",
	Id = "COVERED_BRIDGE_CENTER_TOP_PART",
	Mover = { DataType = "BUILDING_PART_MOVER" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = "COVERED_BRIDGE_CENTER_PREFAB"
	},
	Cost = {
		DataType = "BUILDING_PART_COST",
		RessourcesNeeded = {
			{ Resource = "STONE", Quantity = 5 },
			{ Resource = "PLANK", Quantity = 1 }
		},
	},
})

-- Register the center root part (the scalable part that places multiple center top parts)
coveredBridge:register({
	DataType = "BUILDING_PART",
	Id = "COVERED_BRIDGE_CENTER_CORE_PART",
	Mover = { DataType = "BUILDING_PART_MOVER" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER",
		CoreObjectPrefab = "COVERED_BRIDGE_CENTER_ROOT_PREFAB",
		EndPart = "COVERED_BRIDGE_CENTER_TOP_PART",
		FillerList = {
			"COVERED_BRIDGE_TILING_PART"
		},
		BasementScale = 2,
	},
	Cost = {
		DataType = "BUILDING_PART_COST",
		RessourcesNeeded = {
			{ Resource = "STONE", Quantity = 2 }
		},
	},
})

-- Register the core part (the part with the bridge mover)
coveredBridge:register({
	DataType = "BUILDING_PART",
	Id = "COVERED_BRIDGE_CORE_PART",
	Name = "COVERED_BRIDGE_CORE_NAME",
	Mover = { DataType = "BUILDING_PART_MOVER_BRIDGE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_BRIDGE",
		StartPart = "COVERED_BRIDGE_START_PART",
		EndPart = "COVERED_BRIDGE_END_PART",
		FillerList = {
			"COVERED_BRIDGE_CENTER_CORE_PART"
		},
		SlopeFillerList = {
			"COVERED_BRIDGE_SLOPE_END_PART"
		},
		SlopeStartFillerList = {
			"COVERED_BRIDGE_SLOPE_START_PART"
		},
		MinimumScale = 1,
	},
	BuildingFunction = {
		DataType = "BUILDING_FUNCTION_BRIDGE"
	}
})

-- Register the house decoration part
coveredBridge:register({
	DataType = "BUILDING_PART",
	Id = "COVERED_BRIDGE_HOUSE_PART",
	Name = "COVERED_BRIDGE_HOUSE_NAME",
	Description = "COVERED_BRIDGE_HOUSE_DESC",
	Category = "DECORATION",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = "COVERED_BRIDGE_HOUSE_PREFAB"
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "PLANK", Quantity = 5 }
		}
	}
})

-- Register the workplace decoration part
coveredBridge:register({
	DataType = "BUILDING_PART",
	Id = "COVERED_BRIDGE_TAILOR_PART",
	Name = "COVERED_BRIDGE_TAILOR_NAME",
	Description = "COVERED_BRIDGE_TAILOR_DESC",
	Category = "DECORATION",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = "COVERED_BRIDGE_TAILOR_PREFAB"
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 1 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "PLANK", Quantity = 5 }
		}
	},
	BuildingFunction = {
		DataType = "BUILDING_FUNCTION_MARKET",
                Type = "GOODS",
                CapacityPerSlot = 20,
		WorkerCapacity = 1,
                RelatedJob = { Job = "TAILOR", Behavior = "WORK_BEHAVIOR" },
                InputInventoryCapacity = {
			{ Resource = "WOOLEN_CLOTH", Quantity = 20 }
                },
                ResourceListNeeded = {
			{ Resource = "WOOLEN_CLOTH", Quantity = 1 }
                },
                ResourceProduced = {
			{ Resource = "CLOTHES", Quantity = 1 }
                }
	}

})

-- Register the stone gate set door part
coveredBridge:register({
	DataType = "BUILDING_PART",
	Id = "COVERED_BRIDGE_DOOR_PART",
	Name = "COVERED_BRIDGE_DOOR_NAME",
	Description = "COVERED_BRIDGE_DOOR_DESC",
	Category = "DOOR",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = "COVERED_BRIDGE_DOOR_PREFAB"
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 1 }
		}
	}
})


-- Register the center coverage decoration part
coveredBridge:register({
	DataType = "BUILDING_PART",
	Id = "COVERED_BRIDGE_CENTER_COVERAGE_PART",
	Name = "COVERED_BRIDGE_CENTER_COVERAGE_NAME",
	Description = "COVERED_BRIDGE_CENTER_COVERAGE_DESC",
	Category = "DECORATION",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = "COVERED_BRIDGE_CENTER_COVERAGE_PREFAB"
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "PLANK", Quantity = 5 }
		}
	}
})

-- Register the start or end coverage decoration part
coveredBridge:register({
	DataType = "BUILDING_PART",
	Id = "COVERED_BRIDGE_START_END_COVERAGE_PART",
	Name = "COVERED_BRIDGE_START_END_COVERAGE_NAME",
	Description = "COVERED_BRIDGE_START_END_COVERAGE_DESC",
	Category = "DECORATION",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = "COVERED_BRIDGE_START_END_COVERAGE_PREFAB"
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "PLANK", Quantity = 2 }
		}
	}
})

--------------------------------------------------------------------------

-- Override building part components properties (generated by the building asset processor) to add building zones

coveredBridge:registerPrefabComponent("models/coveredBridge.fbx/Prefab/coveredBridge/CenterPart", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 2, 2 }
})
coveredBridge:registerPrefabComponent("models/coveredBridge.fbx/Prefab/coveredBridge/StartPart", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 2, 2 }
})
coveredBridge:registerPrefabComponent("models/coveredBridge.fbx/Prefab/coveredBridge/EndPart", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 2, 2 }
})
coveredBridge:registerPrefabComponent("models/coveredBridge.fbx/Prefab/coveredBridge/StartSlopePart", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 2, 2 }
})
coveredBridge:registerPrefabComponent("models/coveredBridge.fbx/Prefab/coveredBridge/EndSlopePart", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 2, 2 }
})

coveredBridge:registerPrefabComponent("models/coveredBridgeDecoration.fbx/Prefab/coveredBridge/CenterCoveragePart", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = "MINOR", -- Snaps the decoration only on minor attach nodes
	BuildingZone = { 4, 4 }
})

coveredBridge:registerPrefabComponent("models/coveredBridgeDecoration.fbx/Prefab/coveredBridge/StartEndCoveragePart", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = "MINOR", -- Snaps the decoration only on minor attach nodes
	BuildingZone = { 4, 4 }
})

coveredBridge:registerPrefabComponent("models/coveredBridgeDecoration.fbx/Prefab/coveredBridge/HousePart", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = "MINOR", -- Snaps the decoration only on minor attach nodes
	BuildingZone = { 4, 4 }
})

coveredBridge:registerPrefabComponent("models/coveredBridgeDecoration.fbx/Prefab/coveredBridge/TailorPart", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = "MINOR", -- Snaps the decoration only on minor attach nodes
	BuildingZone = { 4, 4 }
})

coveredBridge:registerPrefabComponent("models/coveredBridgeDecoration.fbx/Prefab/coveredBridge/DecorationDoorPart", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = "DOOR", -- Allows the door to be used as entrance
	HasBuildingZone = true,
	BuildingZone = { 1, 1 }
})

--------------------------------------------------------------------------

-- Configure all BRIDGE colliders

coveredBridge:configurePrefabFlagList("models/coveredBridge.fbx/Prefab/coveredBridge/CenterPart", { "BRIDGE" })
coveredBridge:configurePrefabFlagList("models/coveredBridge.fbx/Prefab/coveredBridge/StartPart", { "BRIDGE" })
coveredBridge:configurePrefabFlagList("models/coveredBridge.fbx/Prefab/coveredBridge/EndPart", { "BRIDGE" })
coveredBridge:configurePrefabFlagList("models/coveredBridge.fbx/Prefab/coveredBridge/StartSlopePart", { "BRIDGE" })
coveredBridge:configurePrefabFlagList("models/coveredBridge.fbx/Prefab/coveredBridge/EndSlopePart", { "BRIDGE" })